#version 430

#define VOXEL_AREA 128 //[32 64 128]
#define VOXEL_RADIUS (VOXEL_AREA/2)
	
#if VOXEL_AREA == 128
	const ivec3 workGroups = ivec3(16, 16, 128); 
#endif
#if VOXEL_AREA == 64
	const ivec3 workGroups = ivec3(8, 8, 64); 
#endif
#if VOXEL_AREA == 32
	const ivec3 workGroups = ivec3(4, 4, 32); 
#endif

uniform int frameCounter;

layout (local_size_x = 8, local_size_y = 8, local_size_z = 1) in;


//to read/write to a 3d image
//layout (r32ui) uniform uimage3D c_image1;

#if FLOODFILL_LIGHTING == 1
	//our 3d image with voxel data
	uniform usampler3D cSampler1;
	layout (r32ui) uniform uimage3D cimage1;
	
	//buffer 2, where we read 
	uniform sampler3D cSampler2_colored_light;
	layout (rgba8) uniform image3D cimage2_colored_light;

	//buffer 3, where  write for later 
	uniform sampler3D cSampler3_colored_light;
	layout (rgba8) uniform image3D cimage3_colored_light;
	
	uniform ivec3 cameraPositionInt;
	uniform ivec3 previousCameraPositionInt;
	uniform vec3 cameraPosition;
	uniform vec3 previousCameraPosition;

#endif

void main()
{
	#if FLOODFILL_LIGHTING == 1
	    //get index of thread
        ivec3 voxel_pos_new = ivec3(gl_GlobalInvocationID.xyz);
		
		//positions
		ivec3 camshift = cameraPositionInt-previousCameraPositionInt;
		
        ivec3 voxel_pos = voxel_pos_new +camshift;
		
			//load data and unpack
			vec4 voxel_data = unpackUnorm4x8( imageLoad(cimage1, voxel_pos_new ).r); 
			
			vec4 old_voxel_data = imageLoad(cimage2_colored_light, voxel_pos ); 
			
		
			//do stuff
			voxel_data=max(voxel_data,old_voxel_data*.9);
					
			#define LIGHT_FADE_VXFL -.1
					
			//floodfill
			ivec3 neighbor = ivec3(1.,0.,0.);
			vec4 bytes = //texture3D(cSampler2_colored_light, vec3(voxel_pos+neighbor)/vec3(VOXEL_AREA));
			imageLoad(cimage2_colored_light, voxel_pos +neighbor);
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			
			neighbor = ivec3(-1.,0.,0.);
			bytes = imageLoad(cimage2_colored_light, voxel_pos +neighbor);	
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			
			neighbor = ivec3(0.,0,1.);
			bytes = imageLoad(cimage2_colored_light, voxel_pos +neighbor);		
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			
			neighbor = ivec3(0.,0,-1.);
			bytes = imageLoad(cimage2_colored_light, voxel_pos +neighbor);	
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			
			neighbor = ivec3(0.,1.,0.);
			bytes = imageLoad(cimage2_colored_light, voxel_pos +neighbor);		
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			
			neighbor = ivec3(0.,-1.,0.);
			bytes = imageLoad(cimage2_colored_light, voxel_pos +neighbor);		
			voxel_data=clamp(max(voxel_data,bytes LIGHT_FADE_VXFL),0.,1.);
			

			//pack data
			//uint integerValue = packUnorm4x8( voxel_data );
			
			//write data
			imageStore(cimage3_colored_light, voxel_pos_new, voxel_data);

    #endif
}

